/*
    dsproxy_mixer.c - OSS trampoline device driver

    Copyright (C) 2000 Ram Sivrasubramanium <ram@linux-workshop.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define __NO_VERSION__

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/string.h>
#include <linux/pci.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <asm/poll.h>
#include "dsproxy.h"


static struct dsproxy_s *get_user_struct(void)
{
struct dsproxy_s *emix;

down(&global_sem);
emix=emhead;
while(emix)
        {
        if(emix->uid==current->uid) break;
        emix=emix->next;
        }

/* the up for this semaphore is placed next to the file_operations return() for mixer */
/* this is needed because the reader end of the pipe may unexpectedly go away on us!  */
return(emix);
}


int dsproxy_mixer_ioctl(struct file *file, struct dsproxy_s *emix, unsigned int cmd, unsigned long arg)
{
int err, len = 0;
int val;

if (_SIOC_DIR(cmd) != _SIOC_NONE && _SIOC_DIR(cmd) != 0) 
	{
	/*
	 * Have to validate the address given by the process.
         */
	len = _SIOC_SIZE(cmd);
	if (len < 1 || len > 65536 || arg == 0) return(-EFAULT);
	if ((_SIOC_DIR(cmd) & _SIOC_WRITE) && ((err = verify_area(VERIFY_READ, (void *)arg, len)) < 0)) return(err);
	if ((_SIOC_DIR(cmd) & _SIOC_READ) && ((err = verify_area(VERIFY_WRITE, (void *)arg, len)) < 0)) return(err);
    	}

if(!(emix=get_user_struct()))
	{
	up(&global_sem);
	return(-ENXIO);
	}

if(get_user(val, (int *)arg)) 
	{
	up(&global_sem);
	return(-EFAULT);
	}

switch(cmd) 
	{ 
    	case SOUND_MIXER_READ_CAPS:
	case SOUND_MIXER_READ_RECMASK:
	case SOUND_MIXER_READ_RECSRC:
		val=0;						/* no capabilities */
		break;

	case SOUND_MIXER_READ_DEVMASK:
    	case SOUND_MIXER_READ_STEREODEVS:
		val=SOUND_MASK_VOLUME | SOUND_MASK_BASS | SOUND_MASK_TREBLE | SOUND_MASK_PCM | SOUND_MASK_OGAIN;
		break;

	case SOUND_MIXER_READ_VOLUME:
		down(&emix->mixer_sem);
		val=emix->volume;
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_READ_BASS:
		down(&emix->mixer_sem);
		val=emix->bass;
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_READ_TREBLE:
		down(&emix->mixer_sem);
		val=emix->treble;
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_READ_PCM:
		down(&emix->mixer_sem);
		val=emix->pcm;
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_READ_OGAIN:
		down(&emix->mixer_sem);
		val=emix->ogain;
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_WRITE_VOLUME:
		down(&emix->mixer_sem);
		if(emix->volume!=val)
			{
			emix->volume=val;
			emix->dirty|=SOUND_MASK_VOLUME;
			}
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_WRITE_BASS:
		down(&emix->mixer_sem);
		if(emix->bass!=val)
			{
			emix->bass=val;
			emix->dirty|=SOUND_MASK_BASS;
			}
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_WRITE_TREBLE:
		down(&emix->mixer_sem);
		if(emix->treble!=val)
			{
			emix->treble=val;
			emix->dirty|=SOUND_MASK_TREBLE;
			}
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_WRITE_PCM:
		down(&emix->mixer_sem);
		if(emix->pcm!=val)
			{
			emix->pcm=val;
			emix->dirty|=SOUND_MASK_PCM;
			}
		up(&emix->mixer_sem);
		break;

	case SOUND_MIXER_WRITE_OGAIN:
		down(&emix->mixer_sem);
		if(emix->ogain!=val)
			{
			emix->ogain=val;
			emix->dirty|=SOUND_MASK_OGAIN;
			}
		up(&emix->mixer_sem);
		break;

    	default:
		up(&global_sem);
		return(-ENXIO);
    	}

up(&global_sem);
return(put_user(val, (int *)arg));
}


int dsproxy_mixer_open(struct inode* inode, struct file* filp) 
{
struct dsproxy_s *rc=get_user_struct();

up(&global_sem);
if(!rc)
	{
	return(-EACCES);
	}

DEBUG(printk("dsproxy_mixer.o: Opening mixer device for uid %d (dsproxy_s -> %p)\n",current->uid,filp->private_data));
return(0);
}


int dsproxy_mixer_release(struct dsproxy_s *em, struct file *filp, int subdevice)
{
DEBUG(printk("dsproxy_mixer.o: Releasing mixer device for uid %d (dsproxy_s -> %p)\n",current->uid,em));
return(0);
}
